#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

#include <iostream>

#include "signal-file.h"

#define DEFAULT_DEVICE "/dev/lirc0"
#define DEFAULT_INTER_SIGNAL_PAUSE 200 // ms

using namespace std;

void showUsage()
{
	cout << "iroutput [options] file command [command...]" << endl;
	cout << endl;
	cout << "options" << endl;
	cout << " -d device                      specify device to use (default " << DEFAULT_DEVICE << ")" << endl;
	cout << " -h                             print this message and exit" << endl;
	cout << " -p pause-length                set the inter-signal pause duration in " << endl;
	cout << "                                milliseconds (default " << DEFAULT_INTER_SIGNAL_PAUSE << ")" << endl;
	cout << " -n repeat-count                set the default signal repeat count (default 1)" << endl;
	cout << endl;
	cout << "command" << endl;
	cout << " signal-name                    output signal once" << endl;
	cout << " signal-name repeat-count       output signal followed by repeat-count" << endl;
	cout << "                                repeat signals" << endl;
	cout << " signal-name -r repeat-count    as above, useful when using numeric signal names" << endl;
	cout << endl;
	cout << "example" << endl;
	cout << " iroutput -d /dev/lirc0 device.txt play volume_up 2 next 1 pause" << endl;
	cout << endl;
}

typedef struct
{
	Signal* signal;
	int repeatCount;
}
SignalT;

bool getInt(const char* str, int* n)
{
	return sscanf(str, "%i", n);
}

int main(int argc, char** argv)
{
	const char* device = DEFAULT_DEVICE;
	int interSignalPause = DEFAULT_INTER_SIGNAL_PAUSE * 1000; // us
	int defaultRepeatCount = 1;
	
	SignalFile signalFile;
	bool haveFile = false;
	
	vector<SignalT> signals;
	
	Signal* signal = NULL;
	Signal* lastSignal = NULL;
	int fd = -1;
	int repeatCount;
	int forceRepeat = false;
	
	
	if(argc == 1)
	{
		showUsage();
		return 0;
	}
	
	// parse args
	for(int i = 1; i < argc; ++i)
	{
		string arg(argv[i]);
		
		if(arg[0] == '-')
		{ // option
			if(arg == "-h")
			{
				showUsage();
				return 0;
			}
			
			// expect parameter
			++i;
			if(i == argc)
			{
				cerr << "Expected parameter for option " << arg << endl;
				return -1;
			}

			if(arg == "-d")
			{
				device = argv[i];
				cout << "Will output using device " << device << endl;
			}
			else
			{
				// expect parameter to be an integer
				int n;
				if(!getInt(argv[i], &n))
				{
					cerr << "Expected integer argument for option " << arg << ", got " << argv[i] << endl;
					return -1;
				}
				
				switch(arg[1])
				{
					case 'p':
						interSignalPause = n * 1000; // us
						cout << "Using inter-signal pause of " << n << " milliseconds" << endl;
						break;
					case 'n':
						defaultRepeatCount = n;
						cout << "Using default repeat count of " << defaultRepeatCount << endl;
						break;
					case 'r':
						if(signals.empty())
						{
							cerr << "Encountered repeat before first signal";
							return -1;
						}
						else
						{
							signals[signals.size() - 1].repeatCount = n;
						}
						break;
					default:
						cerr << "Unrecognised parameter " << arg << endl;
						return -1;
						break;
				}
			}
		}
		else
		{
			// file, command or repeat
			if(!haveFile)
			{
				// file
				if(!signalFile.loadFile(arg)) return -1;
				haveFile = true;
			}
			else
			{
				Signal* signal = signalFile.getSignal(arg);
				if(signal)
				{
					// command
					SignalT signalT;
					signalT.signal = signal;
					signalT.repeatCount = defaultRepeatCount;
					signals.push_back(signalT);
				}
				else
				{
					int repeatCount;
					if(!signals.empty() && getInt(argv[i], &repeatCount))
					{
						// repeat
						signals[signals.size() - 1].repeatCount = repeatCount;
					}
					else
					{
						// ???
						cerr << "Unrecognised signal " << arg << endl;
						return -1;
					}
				}	
			}
		}
	}
	
	if(signals.empty())
	{
		cout << "No signals given" << endl;
		return 0;
	}
	
	// open device
	fd = open(device, O_RDWR);
	if(fd == -1)
	{
		cerr << "Failed to open " << device << " (" << strerror(errno) << ")" << endl;
		return -1;
	}
	
	// output signals
	for(vector<SignalT>::iterator it = signals.begin(); it != signals.end(); ++it)
	{
		Signal* signal = it->signal;
		int n = it->repeatCount;
		cout << "Sending " << signal->name << " with " << n << " repeats" << endl;
		
		it->signal->toRedRat(fd, 0); // the main part of the signal
		if(n) it->signal->toRedRat(fd, n); // the repeat part of the signal
		usleep(interSignalPause);
	}
	
	// close device
	close(fd);
	
	cout << "Finished" << endl;
	return 0;
}

