#include "mod-signal.h"

#include <sys/ioctl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <sstream>
#include <iostream>

#include "lirc-min.h"

using namespace std;

ModulatedSignal::ModulatedSignal()
{
	sendToggle = true; // will get toggled to false on first send
}

bool ModulatedSignal::fromFile(ifstream& file)
{
	string word;
	file >> word;
	if(word != "frequency")
	{
		cerr << "Expected to read frequency, read " << word << endl;
		return false;
	}
	file >> frequency;
	
	file >> word;
	if(word != "pause")
	{
		cerr << "Expected to read pause, read " << word << endl;
		return false;
	}
	file >> pause;

	if(readIntList(file, "lengths", lengths) <= 0)
	{
		cerr << "Failed to read lengths" << endl;
		return false;
	}
	int result = readIntList(file, "main", mainSignal);
	if(result <= 0)
	{
		cerr << "Failed to read main signal" << endl;
		return false;
	}
	if(result % 2 == 0)
	{
		cerr << "Main signal contains " << result << " entries, expected an odd number" << endl;
		return false;
	}  
	
	// now repeat and/or toggle, in that order
	file >> word;
	if(word == "repeat")
	{
		result = readIntList(file, repeatSignal);
		if(result <= 0)
		{
			cerr << "Failed to read repeat signal";
			return false;
		}
		if(result % 2 == 0)
		{
			cerr << "Repeat signal contains " << result << " entries, expected an odd number" << endl;
			return false;
		}
		file >> word; // toggle or ENDSIG
	}
	
	if(word == "ENDSIG") return true; // no toggle data
	
	if(word != "toggle")
	{
		cerr << "Expected to read toggle, read " << word << endl;
		return false;
	}
	
	toggleBits.clear();
	string line;
	getline(file, line);
	stringstream ss(line);
	ToggleBit toggleBit;
	char slash;
	while(ss.good())
	{
		ss >> toggleBit.bit;
		ss >> toggleBit.length1;
		ss >> slash;
		ss >> toggleBit.length2;
		toggleBits.push_back(toggleBit);
	}
	
	file >> word;
	if(word != "ENDSIG")
	{
		cerr << "Expected to read ENDSIG, read " << word << endl;
		return false;
	}
	
	return true;
}

bool ModulatedSignal::toRedRat(int fd, int repeatCount)
{
	if(!repeatCount) sendToggle = !sendToggle; // calls with repeatCount != 0 should use the same value of sendToggle as previous call
	
	if(frequency > 0)
	{
		if(ioctl(fd, LIRC_SET_SEND_CARRIER, &frequency) == -1)
		{
			cerr << "Warning! Failed to set carrier frequency " << frequency << " (" << strerror(errno) << ")" << endl;
			cerr << "Continuing anyway" << endl;
			// keep going because this doesn't work on older kernels
		}
	}
	
	vector<int> data; // technically vector<lirc_t>, but lirc_t is an int anyway
	vector<int> signal = (((repeatCount > 0) && (!repeatSignal.empty())) ? repeatSignal : mainSignal);
	for(vector<int>::iterator it = signal.begin(); it != signal.end(); ++it)
	{
		data.push_back(lengths[*it]);
	}
	if(sendToggle)
	{
		for(vector<ToggleBit>::iterator it = toggleBits.begin(); it != toggleBits.end(); ++it)
		{
			signal[it->bit] = it->length2;
		}
	}
	int size = data.size() * sizeof(int);
	
	int max = ((repeatCount) ? repeatCount : 1);
	for(int i = 0; i < max; ++i)
	{
		void* addr = &data[0];
		if(write(fd, addr, size) == -1)
		{
			cerr << "Failed to write signal " << name << " to RedRat (" << strerror(errno) << ")" << endl;
			return false;
		}
		usleep(pause);
	}
	
	return true;
}
